﻿using System.Linq;
using System.Security.Cryptography;
using System.Threading.Tasks;
using System.Windows.Input;

namespace ViewModel
{
    public delegate void LoginHandler(bool successfulLogin);
    public delegate void LoginExceptionHandler();

    public class Login : BaseViewModel
    {
        private const int SaltSize = 16;
        private const int IterationCount = 100000;
        private const int HashSize = 20;

        public event LoginHandler OnLogin;
        public event LoginExceptionHandler OnLoginException;

        private string _insertedUserName;

        /// <summary>
        /// Obsahuje vložené uživatelské jméno.
        /// </summary>
        public string InsertedUserName
        {
            get { return _insertedUserName; }
            set
            {
                _insertedUserName = value;
                OnPropertyChangedEvent(nameof(InsertedUserName));
            }
        }

        private string _insertedPassword;

        /// <summary>
        /// Obsahuje vložené heslo.
        /// </summary>
        public string InsertedPassword
        {
            get { return _insertedPassword; }
            set
            {
                _insertedPassword = value;
                OnPropertyChangedEvent(nameof(InsertedPassword));
            }
        }

        private bool _loginError = false;

        /// <summary>
        /// Obsahuje informaci o tom, zda poslední pokus o přihlášení byl úspěšný, nebo ne.
        /// </summary>
        public bool LoginError
        {
            get { return _loginError; }
            set
            {
                _loginError = value;
                OnPropertyChangedEvent(nameof(LoginError));
            }
        }

        private RelayCommand _loginCommand;

        /// <summary>
        /// Příkaz, který se pokusí provést přihlášení.
        /// </summary>
        public ICommand LoginCommand
        {
            get
            {
                return _loginCommand ??
                    (_loginCommand = new RelayCommand(async p => await LoginAsync(),
                    p => !string.IsNullOrEmpty(InsertedUserName) && !string.IsNullOrEmpty(InsertedPassword)));
            }
        }

        private void LoginMethod()
        {
            LoginError = !IsValidUser();

            OnLogin?.Invoke(!LoginError);
        }

        private Task LoginAsync()
        {
            return Task.Factory.StartNew(() => LoginMethod());
        }

        private Model.Password CreateNewPassword(string password)
        {
            var deriveBytes = new Rfc2898DeriveBytes(password, SaltSize, IterationCount);
            var derivedPassword = new Model.Password();
            derivedPassword.Hash = deriveBytes.GetBytes(HashSize);
            derivedPassword.Salt = deriveBytes.Salt;
            return derivedPassword;
        }

        private bool IsValidUser()
        {
            try
            {
                var users = Model.Users.GetUsers();

                foreach (var user in users)
                {
                    if (user.Username == InsertedUserName)
                    {
                        var password = new Rfc2898DeriveBytes(InsertedPassword, user.Password.Salt, IterationCount);

                        if (user.Password.Hash.SequenceEqual(password.GetBytes(HashSize)))
                        {
                            return true;
                        }
                    }
                }
            }
            catch
            {
                OnLoginException?.Invoke();
            }

            return false;
        }
    }
}
